/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dacbasic.h	DAC management with DMA

	When		Who	What
	05/05/18	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined DACH7XX_H_
#define DACH7XX_H_
//----------------------------------------------------------------------

#if (WITH_DAC_X == 1)

static	const dacDesc_t	dacDescArray [] =
{
	// For 2 independent channels (2 DMA)
	#if (WITH_DAC_1 == 1)
	{
		10,										// id
		DAC1,									// Pointer to DAC registers
		{	// DMA channel 1 or DUAL
			DMA_DMA2,							//	DMA number 1 or 2, 0 => unused descriptor
			5,									//	DMA stream number 0..7
			DAC_INT_PRIORITY,					//	Interrupt priority
			0,									//	Mux input (not used)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Config: Direction, mode, prio...
		},
		{	// DMA channel 2
			DMA_DMA2,							//	DMA number 1 or 2, 0 => unused descriptor
			6,									//	DMA stream number 0..7
			DAC_INT_PRIORITY,					//	Interrupt priority
			0,									//	Mux input (not used)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Config: Direction, mode, prio...
		},
	},
	#endif	// WITH_DAC_1

	// For channel 1 only or DUAL
	#if (WITH_DAC_1 == 1)
	{
		11,										// id
		DAC1,									// Pointer to DAC registers
		{	// DMA channel 1 or DUAL
			DMA_DMA1,							//	DMA number 1 or 2, 0 => unused descriptor
			5,									//	DMA stream number 0..7
			DAC_INT_PRIORITY,					//	Interrupt priority
			0,									//	Mux input (not used)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Config: Direction, mode, prio...
		},
		{	// DMA channel 2
			DMA_NONE,							//	DMA number 1 or 2, 0 => unused descriptor
			6,									//	DMA stream number 0..7
			DAC_INT_PRIORITY,					//	Interrupt priority
			0,									//	Mux input (not used)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Config: Direction, mode, prio...
		},
	},
	#endif	// WITH_DAC_1

	// For channel 2 only
	#if (WITH_DAC_1 == 1)
	{
		12,										// id
		DAC1,									// Pointer to DAC registers
		{	// DMA channel 1 or DUAL
			DMA_NONE,							//	DMA number 1 or 2, 0 => unused descriptor
			5,									//	DMA stream number 0..7
			DAC_INT_PRIORITY,					//	Interrupt priority
			0,									//	Mux input (not used)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Config: Direction, mode, prio...
		},
		{	// DMA channel 2
			DMA_DMA1,							//	DMA number 1 or 2, 0 => unused descriptor
			6,									//	DMA stream number 0..7
			DAC_INT_PRIORITY,					//	Interrupt priority
			0,									//	Mux input (not used)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Config: Direction, mode, prio...
		},
	},
	#endif	// WITH_DAC_1
} ;

#endif	// WITH_DAC_X

//----------------------------------------------------------------------
#endif	// DACH7XX_H_
