/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	extibasic.c		GPIO and software interrupts management

	When		Who	What
	02/09/18	ac	Creation
	22/01/20	ac	Port to STM32H7xx mono core
	30/04/21	ac	Add individual lines callback

	Doesn't manage D3 domain wake up

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aakernel.h"

#include	"extibasic.h"

//----------------------------------------------------------------------
//	Get interrupt request number from line number 0..19

#define	EXTI_LINE_COUNT	89

static const uint8_t	extiIrqn [EXTI_LINE_COUNT] =
{
		EXTI0_IRQn,			// 0
		EXTI1_IRQn,			// 1
		EXTI2_IRQn,			// 2
		EXTI3_IRQn,			// 3
		EXTI4_IRQn,			// 4
		EXTI9_5_IRQn,		// 5
		EXTI9_5_IRQn,		// 6
		EXTI9_5_IRQn,		// 7
		EXTI9_5_IRQn,		// 8
		EXTI9_5_IRQn,		// 9
		EXTI15_10_IRQn,		// 10
		EXTI15_10_IRQn,		// 11
		EXTI15_10_IRQn,		// 12
		EXTI15_10_IRQn,		// 13
		EXTI15_10_IRQn,		// 14
		EXTI15_10_IRQn,		// 15
		PVD_AVD_IRQn,		// 16
		RTC_Alarm_IRQn,		// 17
		TAMP_STAMP_IRQn,	// 18
		RTC_WKUP_IRQn,		// 19
		COMP_IRQn,			// 20
		COMP_IRQn,			// 21
		0u,					// 22	I2C1 wakeup
		0u,					// 23	I2C2 wakeup
		0u,					// 24	I2C3 wakeup
		0u,					// 25	I2C4 wakeup
		0u,					// 26	USART1 wakeup
		0u,					// 27	USART2 wakeup
		0u,					// 28	USART3 wakeup
		0u,					// 29	USART6 wakeup
		0u,					// 30	UART4 wakeup
		0u,					// 31	UART5 wakeup
		0u,					// 32	UART7 wakeup
		0u,					// 33	UART8 wakeup
		0u,					// 34	LPUART1 wakeup
		0u,					// 35	LPUART2 wakeup
		0u,					// 36	SPI1 wakeup
		0u,					// 37	SPI2 wakeup
		0u,					// 38	SPI3 wakeup
		0u,					// 39	SPI4 wakeup
		0u,					// 40	SPI5 wakeup
		0u,					// 41	SPI6 wakeup
		0u,					// 42	MDIO wakeup
		0u,					// 43	USB1 wakeup
		0u,					// 44	USB2 wakeup
		0u,					// 45	Reserved
		0u,					// 46	Reserved
		0u,					// 47	LPTIM1 wakeup
		0u,					// 48	LPTIM2 wakeup
		0u,					// 49	LPTIM2 output
		0u,					// 50	LPTIM3 wakeup
		0u,					// 51	LPTIM3 output
		0u,					// 52	LPTIM4 wakeup
		0u,					// 53	LPTIM4 output
		0u,					// 54	SWPMI wakeup
		0u,					// 55	WKUP1
		0u,					// 56	WKUP2
		0u,					// 57	WKUP3
		0u,					// 58	WKUP4
		0u,					// 59	WKUP5
		0u,					// 60	WKUP6
		0u,					// 61	RCC interrupt
		0u,					// 62	I2C4 Event interrupt
		0u,					// 63	I2C4 Error interrupt
		0u,					// 64	LPUART1 global Interrupt
		0u,					// 65	SPI6 interrupt
		0u,					// 66	BDMA CH0 interrupt
		0u,					// 67	BDMA CH1 interrupt
		0u,					// 68	BDMA CH2 interrupt
		0u,					// 69	BDMA CH3 interrupt
		0u,					// 70	BDMA CH4 interrupt
		0u,					// 71	BDMA CH5 interrupt
		0u,					// 72	BDMA CH6 interrupt
		0u,					// 73	BDMA CH7 interrupt
		0u,					// 74	DMAMUX2 interrupt
		0u,					// 75	ADC3 interrupt
		0u,					// 76	SAI4 interrupt
		0u,					// 77	Reserved
		0u,					// 78	Reserved
		0u,					// 79	Reserved
		0u,					// 80	Reserved
		0u,					// 81	Reserved
		0u,					// 82	Reserved
		0u,					// 83	Reserved
		0u,					// 84	Reserved
		CEC_IRQn,			// 85	HDMI-CEC wakeup
		ETH_WKUP_IRQn,		// 86	ETHERNET wakeup
		0u,					// 87	HSECSS interrupt
		0u					// 88	Reserved
} ;

//----------------------------------------------------------------------
// Structures for a group of 32 EXTI lines

typedef struct
{
	__IO uint32_t	RTSR ;
	__IO uint32_t	FTSR ;
	__IO uint32_t	SWIER ;
	__IO uint32_t	D3PMR ;
	__IO uint32_t	D3PCRL ;
	__IO uint32_t	D3PCRH ;
	uint32_t      	RESERVED [2] ;

} extiCfg_t ;

typedef struct
{
	__IO uint32_t	IMR ;
	__IO uint32_t	EMR ;
	__IO uint32_t	PR ;
	uint32_t      	RESERVED ;

} extiItEv_t ;

// A structure to define the registers of all EXTI lines

typedef struct
{
	extiCfg_t	cfg1 ;
	extiCfg_t	cfg2 ;
	extiCfg_t	cfg3 ;
	uint32_t    RESERVED [8] ;
	extiItEv_t	it1 ;
	extiItEv_t	it2 ;
	extiItEv_t	it3 ;

} exti_t ;

//----------------------------------------------------------------------

static	IRQn_Type	lineToIrqn_ (uint32_t lineNumber)
{
	if (lineNumber < EXTI_LINE_COUNT)
	{
		return (IRQn_Type) extiIrqn [lineNumber] ;
	}
	else
	{
		AA_ASSERT (0) ;
		return (IRQn_Type) 0 ;
	}
}

//----------------------------------------------------------------------
//	Configure registers for interrupt or event

static extiError_t	extiSetRegisters_ (uint32_t lineNumber, uint32_t priority, uint32_t flags)
{
	extiCfg_t		* pCfg ;
	extiItEv_t		* pIt  ;
	uint32_t		pinMask ;

	if (lineNumber < 32)
	{
		pinMask = 1u << lineNumber ;
		pCfg = (extiCfg_t *) (& (((exti_t *) EXTI)->cfg1)) ;
		pIt = (extiItEv_t *) (& (((exti_t *) EXTI)->it1)) ;
		if ((pinMask & EXTI_IMR1_IM) == 0u)
		{
			return EXTI_EFAIL ;		// Invalid line number
		}
	}
	else if (lineNumber < 64)
	{
		pinMask = 1u << (lineNumber - 32u) ;
		pCfg = (extiCfg_t *) (& (((exti_t *) EXTI)->cfg2)) ;
		pIt = (extiItEv_t *) (& (((exti_t *) EXTI)->it2)) ;
		if ((pinMask & EXTI_IMR2_IM) == 0u)
		{
			return EXTI_EFAIL ;		// Invalid line number
		}
	}
	else if (lineNumber < 96)
	{
		pinMask = 1u << (lineNumber - 64) ;
		pCfg = (extiCfg_t *) (& (((exti_t *) EXTI)->cfg3)) ;
		pIt = (extiItEv_t *) (& (((exti_t *) EXTI)->it3)) ;
		if ((pinMask & EXTI_IMR3_IM) == 0u)
		{
			return EXTI_EFAIL ;		// Invalid line number
		}
	}
	else
	{
		return EXTI_EFAIL ;		// Invalid line number
	}

	// Mask all events
	pIt->EMR &= ~pinMask ;
	pIt->IMR &= ~pinMask ;

	// Configure trigger active edge
	pCfg->FTSR &= ~pinMask ;
	pCfg->RTSR &= ~pinMask;
	if ((flags & EXTI_FLAG_FALLING) != 0u)
	{
		pCfg->FTSR |= pinMask ;		// Falling
	}
	if ((flags & EXTI_FLAG_RISING) != 0u)
	{
		pCfg->RTSR |= pinMask ;		// Rising
	}

	// Set event flag if required
	if ((flags & EXTI_FLAG_EVENT) != 0u)
	{
		pIt->EMR |= pinMask ;
	}

	// Add to NVIC if interrupt is required
	if ((flags & EXTI_FLAG_INTR) != 0u)
	{
		IRQn_Type irqNum = lineToIrqn_ (lineNumber) ;
		if (irqNum != 0u)
		{
			pIt->IMR |= pinMask ;
			NVIC_EnableIRQ   (irqNum) ;
			NVIC_SetPriority (irqNum, priority) ;
		}
	}

	return EXTI_ENONE  ;
}

//----------------------------------------------------------------------
//	Initialize an interrupt or an event on a GPIO pin
//	Pin: 0..15
//	The pin is configured using pinDesc
//	Allowed flags: EXTI_FLAG_INTR, EXTI_FLAG_EVENT, EXTI_FLAG_RISING, EXTI_FLAG_FALLING

extiError_t	extiInitGpio	(const gpioPinDesc_t * pPinDesc, uint32_t priority, uint32_t flags)
{
	uint32_t		reg ;

	if (pPinDesc->pin > 15)
	{
		AA_ASSERT (0) ;
		return EXTI_EFAIL ;
	}

	if ((flags & EXTI_FLAG_GPIO_NOCONF) == 0)
	{
		gpioConfigurePin (pPinDesc) ;
	}

	// Enable SYSCFG clock
	RCC->APB4ENR |= RCC_APB4ENR_SYSCFGEN ;

	// Set proper GPIO port in SYSCFG->EXTICRn
	reg = SYSCFG->EXTICR [pPinDesc->pin / 4] ;
	reg &= ~(0x0Fu << (0x04u * (pPinDesc->pin & 0x03u))) ;
	reg |= (uint32_t) ((pPinDesc->port - 'A') << (0x04u * (pPinDesc->pin & 0x03u))) ;
	SYSCFG->EXTICR [pPinDesc->pin / 4u] = reg ;

	return extiSetRegisters_ (pPinDesc->pin, priority, flags) ;
}

//----------------------------------------------------------------------
//	Initialize an interrupt or an event on a line 16..EXTI_LINE_COUNT-1
//	Allowed flags for configurable line: EXTI_FLAG_INTR, EXTI_FLAG_EVENT, EXTI_FLAG_RISING, EXTI_FLAG_FALLING
//	Allowed flags for direct line: EXTI_FLAG_INTR, EXTI_FLAG_EVENT

extiError_t	extiInitLine		(uint32_t lineNumber, uint32_t priority, uint32_t flags)
{
	if (lineNumber < 16  ||  lineNumber >= EXTI_LINE_COUNT)
	{
		AA_ASSERT (0) ;
		return EXTI_EFAIL ;
	}

	return extiSetRegisters_ (lineNumber, priority, flags) ;
}

//----------------------------------------------------------------------
//	Initialize an exti line 0..EXTI_LINE_COUNT-1 for software interrupt
//	This doesn't allows to configure a software event

extiError_t	extiInitSwi		(uint32_t lineNumber, uint32_t priority)
{
	if (lineNumber >= EXTI_LINE_COUNT)
	{
		AA_ASSERT (0) ;
		return EXTI_EFAIL ;
	}

	// Force trigger on rising edge
	return extiSetRegisters_ (lineNumber, priority, EXTI_FLAG_INTR | EXTI_FLAG_RISING) ;
}

//----------------------------------------------------------------------

void		extiResetLine	(uint32_t lineNumber)
{
	uint32_t	pinMask ;

	if (lineNumber < 32u)
	{
		pinMask = ~(1u << lineNumber) ;
		EXTI->IMR1	&= pinMask ;
		EXTI->EMR1	&= pinMask ;
		EXTI->FTSR1	&= pinMask ;
		EXTI->RTSR1	&= pinMask ;
	}
	else if (lineNumber < 64u)
	{
		pinMask = ~(1u << (lineNumber - 32u)) ;
		EXTI->IMR2	&= pinMask ;
		EXTI->EMR2	&= pinMask ;
		EXTI->FTSR2	&= pinMask ;
		EXTI->RTSR2	&= pinMask ;
	}
	else if (lineNumber < 96u)
	{
		pinMask = ~(1u << (lineNumber - 64u)) ;
		EXTI->IMR3	&= pinMask ;
		EXTI->EMR3	&= pinMask ;
		EXTI->FTSR3	&= pinMask ;
		EXTI->RTSR3	&= pinMask ;
	}
	else
	{
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// Interrupt handlers

#if (WITH_EXTI_0 == 1)
void EXTI0_IRQHandler (void) ;
void EXTI0_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR0) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR0 ;
		// Call user callback
		extiCallback0 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_1 == 1)
void EXTI1_IRQHandler (void) ;
void EXTI1_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR1) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR1 ;
		// Call user callback
		extiCallback1 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_2 == 1)
void EXTI2_IRQHandler (void) ;
void EXTI2_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR2) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR2 ;
		// Call user callback
		extiCallback2 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_3 == 1)
void EXTI3_IRQHandler (void) ;
void EXTI3_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR3) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR3 ;
		// Call user callback
		extiCallback3 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_4 == 1)
void EXTI4_IRQHandler (void) ;
void EXTI4_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR4) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR4 ;
		// Call user callback
		extiCallback4 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_9_5 == 1)
void EXTI9_5_IRQHandler (void) ;
void EXTI9_5_IRQHandler (void)
{
	uint32_t	reg ;

	aaIntEnter () ;

	reg = EXTI->PR1 >> 5 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR5 ;
		extiCallback5 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR6 ;
		extiCallback6 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR7 ;
		extiCallback7 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR8 ;
		extiCallback8 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR9 ;
		extiCallback9 (EXTI_FLAG_RISING) ;
	}

	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_15_10 == 1)
void EXTI15_10_IRQHandler (void) ;
void EXTI15_10_IRQHandler (void)
{
	uint32_t	reg ;

	aaIntEnter () ;

	reg = EXTI->PR1 >> 10 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR10 ;
		extiCallback10 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR11 ;
		extiCallback11 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR12 ;
		extiCallback12 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;
	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR13 ;
		extiCallback13 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR14 ;
		extiCallback14 (EXTI_FLAG_RISING) ;
	}
	reg >>= 1 ;

	if ((reg & 1) != 0)
	{
		EXTI->PR1 = EXTI_PR1_PR15 ;
		extiCallback15 (EXTI_FLAG_RISING) ;
	}

	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_PVD == 1)
void PVD_AVD_IRQHandler (void) ;
void PVD_AVD_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR16) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR16 ;
		// Call user callback
		extiCallback16 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_RTC_ALRM == 1)
void RTC_Alarm_IRQHandler (void) ;
void RTC_Alarm_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR17) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR17 ;
		// Call user callback
		extiCallback17 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_TAMP_STAMP == 1)
void TAMP_STAMP_IRQHandler (void) ;
void TAMP_STAMP_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR18) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR18 ;
		// Call user callback
		extiCallback18 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_RTC_WKUP == 1)
void RTC_WKUP_IRQHandler (void) ;
void RTC_WKUP_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR19) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR19 ;
		// Call user callback
		extiCallback19 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_COMP == 1)
void COMP1_IRQHandler (void) ;
void COMP1_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR1 & EXTI_PR1_PR20) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR20 ;
		// Call user callback
		extiCallback20 (EXTI_FLAG_RISING) ;
	}
	if ((EXTI->PR1 & EXTI_PR1_PR21) != 0u)
	{
		// Clear bit
		EXTI->PR1 = EXTI_PR1_PR21 ;
		// Call user callback
		extiCallback21 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_LPTIM2 == 1)
void LPTIM2_IRQHandler (void) ;
void LPTIM2_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR2 & EXTI_PR2_PR49) != 0u)
	{
		// Clear bit
		EXTI->PR2 = EXTI_PR2_PR49 ;
		// Call user callback
		extiCallback49 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_LPTIM3 == 1)
void LPTIM3_IRQHandler (void) ;
void LPTIM3_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR2 & EXTI_PR2_PR51) != 0u)
	{
		// Clear bit
		EXTI->PR2 = EXTI_PR2_PR51 ;
		// Call user callback
		extiCallback51 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_CEC == 1)
void CEC_IRQHandler (void) ;
void CEC_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR3 & EXTI_PR3_PR85) != 0u)
	{
		// Clear bit
		EXTI->PR3 = EXTI_PR3_PR85 ;
		// Call user callback
		extiCallback85 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------

#if (WITH_EXTI_ETH == 1)
void ETH_WKUP_IRQHandler (void) ;
void ETH_WKUP_IRQHandler (void)
{
	aaIntEnter () ;
	if ((EXTI->PR3 & EXTI_PR3_PR86) != 0u)
	{
		// Clear bit
		EXTI->PR3 = EXTI_PR3_PR86 ;
		// Call user callback
		extiCallback86 (EXTI_FLAG_RISING) ;
	}
	aaIntExit () ;
}
#endif

//----------------------------------------------------------------------
//	Default callback for all lines

void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback0		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback1		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback2		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback3		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback4		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback5		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback6		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback7		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback8		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback9		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback10		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback11		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback12		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback13		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback14		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback15		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback16		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback17		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback18		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback19		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback20		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback21		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback49		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback51		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback85		(uint32_t flags) ;
void __attribute__ ((weak, alias ("extiDefaultCallback")))	extiCallback86		(uint32_t flags) ;

static	void extiDefaultCallback (uint32_t flags)
{
	(void) flags ;
}

//----------------------------------------------------------------------
