/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	extibasic.c	EXTI and software interrupt

	When		Who	What
	02/09/18	ac	Creation
	22/01/20	ac	Port to STM32H7xx mono core
	30/04/21	ac	Add individual lines callback

	Doesn't manage D3 domain wake up

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined EXTIBASIC_H_
#define	EXTIBASIC_H_
//----------------------------------------------------------------------

// Define which lines interrupt handlers should be managed by extibasic
// This corresponds to "configurable" lines
// Those who are at 0 can have a user defined interrupt handlers
#define	WITH_EXTI_0				0
#define	WITH_EXTI_1				1
#define	WITH_EXTI_2				1
#define	WITH_EXTI_3				1
#define	WITH_EXTI_4				1
#define	WITH_EXTI_9_5			1
#define	WITH_EXTI_15_10			1
#define	WITH_EXTI_PVD			1
#define	WITH_EXTI_RTC_ALRM		1
#define	WITH_EXTI_TAMP_STAMP	1
#define	WITH_EXTI_RTC_WKUP		1
#define	WITH_EXTI_COMP			1
#define	WITH_EXTI_LPTIM2		1
#define	WITH_EXTI_LPTIM3		1
#define	WITH_EXTI_CEC			1
#define	WITH_EXTI_ETH			1

#include	"gpiobasic.h"

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Definition of returned values

typedef enum
{
	EXTI_ENONE		= 0u,
	EXTI_EFAIL		= 1u

} extiError_t ;

// Bits definition for flag argument
// They are not exclusive
#define	EXTI_FLAG_INTR			0x0001u
#define	EXTI_FLAG_EVENT			0x0002u

#define	EXTI_FLAG_RISING		0x0004u
#define	EXTI_FLAG_FALLING		0x0008u

#define	EXTI_FLAG_GPIO_NOCONF	0x8000u		// extiInitGpio() doesn't configure the GPIO pin (the user does it)

// The name of the lines
#define	EXTI_GPIO_0			0
#define	EXTI_GPIO_1			1
#define	EXTI_GPIO_2			2
#define	EXTI_GPIO_3			3
#define	EXTI_GPIO_4			4
#define	EXTI_GPIO_5			5
#define	EXTI_GPIO_6			6
#define	EXTI_GPIO_7			7
#define	EXTI_GPIO_8			8
#define	EXTI_GPIO_9			9
#define	EXTI_GPIO_10		10
#define	EXTI_GPIO_11		11
#define	EXTI_GPIO_12		12
#define	EXTI_GPIO_13		13
#define	EXTI_GPIO_14		14
#define	EXTI_GPIO_15		15
#define	EXTI_PVD			16		// PVD
#define	EXTI_RTC_ALARM		17		// RTC or SSRU for L4Px and L4Qx
#define	EXTI_TAMP_STAMP		18		// RTC tamper or timestamp or CCSS LSE
#define	EXTI_RTC_WKUP		19
#define	EXTI_COMP1			20		// COMP1 output
#define	EXTI_COMP2			21		// COMP2 output
#define	EXTI_I2C1_WKUP		22		// I2C1WKUP
#define	EXTI_I2C2_WKUP		23		// I2C2 wakeup
#define	EXTI_I2C3_WKUP		24		// I2C3 wakeup
#define	EXTI_I2C4_WKUP		25		// I2C4 wakeup
#define	EXTI_USART1_WKUP	26		// USART1 wakeup
#define	EXTI_USART2_WKUP	27		// USART2 wakeup
#define	EXTI_USART3_WKUP	28		// USART3 wakeup
#define	EXTI_USART6_WKUP	29		// USART6 wakeup
#define	EXTI_UART4_WKUP		30		// UART4 wakeup
#define	EXTI_UART5_WKUP		31		// UART5 wakeup
#define	EXTI_UART7_WKUP		32		// UART7 wakeup
#define	EXTI_UART8_WKUP		33		// UART8 wakeup
#define	EXTI_LPUART1_WKUP	34		// LPUART1 wakeup
#define	EXTI_LPUART2_WKUP	35		// LPUART2 wakeup
#define	EXTI_SPI1_WKUP		36		// SPI1 wakeup
#define	EXTI_SPI2_WKUP		37		// SPI2 wakeup
#define	EXTI_SPI3_WKUP		38		// SPI3 wakeup
#define	EXTI_SPI4_WKUP		39		// SPI4 wakeup
#define	EXTI_SPI5_WKUP		40		// SPI5 wakeup
#define	EXTI_SPI6_WKUP		41		// SPI6 wakeup
#define	EXTI_MDIO_WKUP		42		// MDIO wakeup
#define	EXTI_USB1_WKUP		43		// USB1 wakeup
#define	EXTI_USB2_WKUP		44		// USB2 wakeup
#define	EXTI_LPTIM1_WKUP	47		// LPTIM1 wakeup
#define	EXTI_LPTIM2_WKUP	48		// LPTIM2 wakeup
#define	EXTI_LPTIM2_OUT		49		// LPTIM2 output
#define	EXTI_LPTIM3_WKUP	50		// LPTIM3 wakeup
#define	EXTI_LPTIM3_OUT		51		// LPTIM3 output
#define	EXTI_LPTIM4_WKUP	52		// LPTIM4 wakeup
#define	EXTI_LPTIM4_OUT		53		// LPTIM4 output
#define	EXTI_SWPMI_WKUP		54		// SWPMI wakeup
#define	EXTI_WKUP1			55		// WKUP1
#define	EXTI_WKUP2			56		// WKUP2
#define	EXTI_WKUP3			57		// WKUP3
#define	EXTI_WKUP4			58		// WKUP4
#define	EXTI_WKUP5			59		// WKUP5
#define	EXTI_WKUP6			60		// WKUP6
#define	EXTI_RCC			61		// RCC interrupt
#define	EXTI_I2C4_EV		62		// I2C4 Event interrupt
#define	EXTI_I2C4_ER		63		// I2C4 Error interrupt
#define	EXTI_LPUART1		64		// LPUART1 global Interrupt
#define	EXTI_SPI6			65		// SPI6 interrupt
#define	EXTI_BDMA_CH0		66		// BDMA CH0 interrupt
#define	EXTI_BDMA_CH1		67		// BDMA CH1 interrupt
#define	EXTI_BDMA_CH2		68		// BDMA CH2 interrupt
#define	EXTI_BDMA_CH3		69		// BDMA CH3 interrupt
#define	EXTI_BDMA_CH4		70		// BDMA CH4 interrupt
#define	EXTI_BDMA_CH5		71		// BDMA CH5 interrupt
#define	EXTI_BDMA_CH6		72		// BDMA CH6 interrupt
#define	EXTI_BDMA_CH7		73		// BDMA CH7 interrupt
#define	EXTI_DMAM_UX2		74		// DMAMUX2 interrupt
#define	EXTI_ADC3			75		// ADC3 interrupt
#define	EXTI_SAI4			76		// SAI4 interrupt
#define	EXTI_CEC_WKUP		85		// HDMI-CEC wakeup
#define	EXTI_ETH_WKUP		86		// ETHERNET wakeup
#define	EXTI_HSECSS			87		// HSECSS interrupt

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#ifdef __cplusplus
extern "C" {
#endif

// The parameter lineNumber is 0..15  for extiInitGpio()
// The parameter lineNumber is 16..87 for extiInitLine()
// The parameter lineNumber is 0..87  for extiInitSwi(), configurable line only (not direct line)
// The parameter lineNumber is 0..87  for extiResetLine()

extiError_t	extiInitGpio		(const gpioPinDesc_t * pPinDesc, uint32_t priority, uint32_t flags) ;
extiError_t	extiInitLine		(uint32_t lineNumber, uint32_t priority, uint32_t flags) ;
extiError_t	extiInitSwi			(uint32_t lineNumber, uint32_t priority) ;
void		extiResetLine		(uint32_t lineNumber) ;

// This function generate a software interrupt on the specified line
// Valid only for "configurable" EXTI lines (not "direct" lines)
__STATIC_INLINE void extiSwiTrigger	(uint32_t lineNumber)
{
	volatile uint32_t	* pSwier ;

	pSwier = (uint32_t *) ((uint32_t) (& EXTI->SWIER1) + (lineNumber & ~0x1Fu)) ;
	* pSwier = (uint32_t) (1u << (lineNumber - (lineNumber & ~0x1Fu))) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// The EXTI callback function type
// These functions are called from library interrupt handler each time an interrupt occurs on EXTI line.
// flags: combination of EXTI_FLAG_RISING and EXTI_FLAG_FALLING
// On F4/L4/H7 families there is only one event so EXTI_FLAG_RISING is used for every event
typedef void (* extiCallaback_t) (uint32_t flags) ;

/*
	When you use an EXTI line interrupt,
	you must have a callback function whose name is build from the word "extiCallback" concatenated to the line number.
	Example: for the line 12 the callback name is extiCallback12.
	The macro EXTI_CALLBACK help to build the callback name.

	#define	MY_EXTI_LINE			12

	Then use your define to write the callback function:

	void EXTI_CALLBACK(MY_EXTI_LINE) (uint32_t flags)
	{
	}

	Using the EXTI_CALLBACK macro, when you change the EXTI line number,
	you don't have to change the callback name.
*/
#define	EXTI_CB_BUILDER(x)		extiCallback ## x
#define	EXTI_CALLBACK(x)		EXTI_CB_BUILDER(x)


// The EXTI callbacks for configurable EXTI lines
// The user should define its own functions with these names
void	extiCallback0		(uint32_t flags) ;
void	extiCallback1		(uint32_t flags) ;
void	extiCallback2		(uint32_t flags) ;
void	extiCallback3		(uint32_t flags) ;
void	extiCallback4		(uint32_t flags) ;
void	extiCallback5		(uint32_t flags) ;
void	extiCallback6		(uint32_t flags) ;
void	extiCallback7		(uint32_t flags) ;
void	extiCallback8		(uint32_t flags) ;
void	extiCallback9		(uint32_t flags) ;
void	extiCallback10		(uint32_t flags) ;
void	extiCallback11		(uint32_t flags) ;
void	extiCallback12		(uint32_t flags) ;
void	extiCallback13		(uint32_t flags) ;
void	extiCallback14		(uint32_t flags) ;
void	extiCallback15		(uint32_t flags) ;
void	extiCallback16		(uint32_t flags) ;
void	extiCallback17		(uint32_t flags) ;
void	extiCallback18		(uint32_t flags) ;
void	extiCallback19		(uint32_t flags) ;
void	extiCallback20		(uint32_t flags) ;
void	extiCallback21		(uint32_t flags) ;
void	extiCallback49		(uint32_t flags) ;
void	extiCallback51		(uint32_t flags) ;
void	extiCallback85		(uint32_t flags) ;
void	extiCallback86		(uint32_t flags) ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// EXTIBASIC_H_
