/*
----------------------------------------------------------------------
	
	AdAstra - Real Time Kernel

	Alain Chebrou

	test.c		Kernel and peripheral libraries tests and example

	When		Who	What
	25/11/13	ac	Creation

----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aalogmes.h"
#include	"aaprintf.h"
#include	"itm.h"

#include	"test.h"

//--------------------------------------------------------------------------------
//	Called from the first user task

void	tests ()
{
	double		f1, f2 ;
	char		cc ;

	#if (AA_WITH_TRACE == 1)
	{
		// If traces are allowed by AA_WITH_TRACE, then enable trace (default is disable)
		aaTraceEnable (1u) ;
	}
	#endif

	aaTaskDelay (1u)  ;	// Allows low level tasks to start

	// Rise this thread priority
	aaTaskSetPriority (AA_SELFTASKID, 14u) ;

	// Display amount of free space
	#if (AA_WITH_MALLOC_TLSF == 1)
	{
		// Display dynamic memory usage
		uint32_t	count, size, used ;

		aaMallocStat (& size, & used, & count) ;
		aaPrintf ("aaMallocStat size %u, used %u, count %u\n", size, used, count) ;
	}
	#endif

	f1 = 1.5 ;
	f2 = 0.5 ;
	f2 += f1 ;
	aaPrintf ("f2= %d (expected 200)\n", (uint32_t) (f2*100)) ;

	while (1)
	{
		aaPrintf ("Select test h for help: ") ;
		cc = aaGetChar () ;
		aaPutChar (cc) ;
		aaPutChar ('\n') ;

		if (cc == 'h')
		{
			aaPrintf ("d  DAC test\n")  ;
			aaPrintf ("m  MCO test\n")  ;
			aaPrintf ("s  SPI test\n")  ;
			aaPrintf ("i  I2C test\n")  ;
			aaPrintf ("t  TIM test\n")  ;
			aaPrintf ("u  USB test\n")  ;
			aaPrintf ("w  SWI test\n")  ;
			aaPrintf ("z  ZLI test\n")  ;
			continue ;
		}

		if (cc == 'q')
		{
			break ;	// End of test
		}

		switch (cc)
		{
			case 'd':
				dacTest () ;
				break ;

			case 'm':
				mcoTest () ;
				break ;

			case 's':
				spiTest () ;
				break ;

			case 'i':
				i2cTest () ;
				break ;

			case 't':
				timTest () ;
				break ;

			case 'u':
				usbTest () ;
				break ;

			case 'w':
				swiTest () ;
				break ;

			case 'z':
				zliTest () ;
				break ;

			default:
				// Use aaLogMes() to see if SWO is enabled (if DEBUG and running with debugger)
				aaLogMes ("log! %c\n", cc, 0, 0, 0, 0) ;
				break ;
		}

		// Some float computing
		f2 += 1.0 ;
		aaPrintf ("f2= %d\n", (uint32_t) (f2*100)) ;
	}
	aaPrintf ("End of test\n") ;
}

//--------------------------------------------------------------------------------
