/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	testmco.c		Configure MCO1/2 output for STM32H7xx

	When		Who	What
	09/04/20	ac	Creation

----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aalogmes.h"
#include	"aaprintf.h"

#include	"test.h"
#include	"gpiobasic.h"

#include	"stm32h7xx_ll_rcc.h"
#include	"stm32h7xx_ll_pwr.h"

//--------------------------------------------------------------------------------
//	Interactive selection of MCO1 / MCO2

static const uint32_t mco1Source [5] =
{
	LL_RCC_MCO1SOURCE_HSI,
	LL_RCC_MCO1SOURCE_LSE,
	LL_RCC_MCO1SOURCE_HSE,
	LL_RCC_MCO1SOURCE_PLL1QCLK,
	LL_RCC_MCO1SOURCE_HSI48,
} ;

static const uint32_t mco2Source [6] =
{
	LL_RCC_MCO2SOURCE_SYSCLK,
	LL_RCC_MCO2SOURCE_PLL2PCLK,
	LL_RCC_MCO2SOURCE_HSE,
	LL_RCC_MCO2SOURCE_PLL1PCLK,
	LL_RCC_MCO2SOURCE_CSI,
	LL_RCC_MCO2SOURCE_LSI
} ;

static	uint32_t	mco1Sel ;
static	uint32_t	mco2Sel ;

static	uint32_t	mco1Div ;
static	uint32_t	mco2Div ;

//----------------------------------------------------------------------

void	configureMco (uint32_t co)
{
	uint32_t	cc ;
RCC_TypeDef * rcc = RCC ;
(void) rcc ;

	if (co == '1')
	{
		// Configure MCO1 output
		gpioPinDesc_t	io = { 'A', 8, AA_GPIO_AF_0, AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_VERYHIGH | AA_GPIO_PUSH_PULL } ;

		gpioConfigurePin (& io) ;
		mco1Div = ((RCC->CFGR & RCC_CFGR_MCO1PRE) >> RCC_CFGR_MCO1PRE_Pos) ;
		mco1Sel = ((RCC->CFGR & RCC_CFGR_MCO1) >> RCC_CFGR_MCO1_Pos) + '0' ;

		while (1)
		{
			aaPrintf ("0   HSI\n") ;
			aaPrintf ("1   LSE\n") ;
			aaPrintf ("2   HSE\n") ;
			aaPrintf ("3   PLL1Q\n") ;
			aaPrintf ("4   HSI48\n") ;
			aaPrintf ("d   Disable and quit\n") ;
			aaPrintf ("+ - Set divisor\n") ;
			aaPrintf ("q   quit\n") ;
			aaPrintf ("Select source %c / %u: ", mco1Sel, mco1Div+1) ;
			cc = aaGetChar () ;
			aaPutChar ((char) cc) ;
			aaPutChar ('\n') ;

			if (cc == 'q')
			{
				break ;
			}

			if (cc >= '0'  && cc <= '4')
			{
				mco1Sel = cc ;
				RCC->CFGR = ((RCC->CFGR & ~RCC_CFGR_MCO1) | (mco1Source [cc - '0'])) ;
			}
			else if (cc == 'd')
			{
				// Disable MCO
				gpioPinDesc_t	iod = { 'A', 8u, 0u, AA_GPIO_MODE_ANALOG } ;
				gpioConfigurePin (& iod) ;
				mco1Sel = cc ;
				break ;
			}
			else if (cc == '+')
			{
				if (mco1Div < 15)
				{
					mco1Div++ ;
					RCC->CFGR = ((RCC->CFGR & ~RCC_CFGR_MCO1PRE) | (mco1Div << RCC_CFGR_MCO1PRE_Pos)) ;
				}
				aaPrintf ("MCO1 divider: %u\n", mco1Div) ;
			}
			else if (cc == '-')
			{
				if (mco1Div > 0u)
				{
					mco1Div-- ;
					RCC->CFGR = ((RCC->CFGR & ~RCC_CFGR_MCO1PRE) | (mco1Div << RCC_CFGR_MCO1PRE_Pos)) ;
				}
				aaPrintf ("MCO1 divider: %u\n", mco1Div) ;
			}
		}
	}
	else
	{
		// Configure MCO2 output
		gpioPinDesc_t	io = { 'C', 9, AA_GPIO_AF_0, AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_VERYHIGH | AA_GPIO_PUSH_PULL } ;

		gpioConfigurePin (& io) ;
		mco2Div = ((RCC->CFGR & RCC_CFGR_MCO2PRE) >> RCC_CFGR_MCO2PRE_Pos) ;
		mco2Sel = ((RCC->CFGR & RCC_CFGR_MCO2) >> RCC_CFGR_MCO2_Pos) + '0' ;

		while (1)
		{
			aaPrintf ("0   SYSCLK\n") ;
			aaPrintf ("1   PLL2P\n") ;
			aaPrintf ("2   HSE\n") ;
			aaPrintf ("3   PLL1P\n") ;
			aaPrintf ("4   CSI\n") ;
			aaPrintf ("5   LSI\n") ;
			aaPrintf ("d   Disable and quit\n") ;
			aaPrintf ("+ - Set divisor\n") ;
			aaPrintf ("q   Quit\n") ;
			aaPrintf ("Select source %c / %u: ", mco2Sel, mco2Div+1) ;
			cc = aaGetChar () ;
			aaPutChar ((char) cc) ;
			aaPutChar ('\n') ;

			if (cc == 'q')
			{
				break ;
			}

			if (cc >= '0'  && cc <= '5')
			{
				mco2Sel = cc ;
				RCC->CFGR = ((RCC->CFGR & ~RCC_CFGR_MCO2) | (mco2Source [cc - '0'])) ;
			}
			else if (cc == 'd')
			{
				// Disable MCO
				gpioPinDesc_t	iod = { 'C', 9, 0, AA_GPIO_MODE_ANALOG } ;
				gpioConfigurePin (& iod) ;
				mco2Sel = cc ;
				break ;
			}
			else if (cc == '+')
			{
				if (mco2Div < 15)
				{
					mco2Div++ ;
					RCC->CFGR = ((RCC->CFGR & ~RCC_CFGR_MCO2PRE) | (mco2Div << RCC_CFGR_MCO2PRE_Pos)) ;
				}
				aaPrintf ("MCO2 divider: %u\n", mco2Div) ;
			}
			else if (cc == '-')
			{
				if (mco2Div > 0)
				{
					mco2Div-- ;
					RCC->CFGR = ((RCC->CFGR & ~RCC_CFGR_MCO2PRE) | (mco2Div << RCC_CFGR_MCO2PRE_Pos)) ;
				}
				aaPrintf ("MCO2 divider: %u\n", mco2Div) ;
			}
		}
	}
}

//--------------------------------------------------------------------------------

void	mcoTest	(void)
{
	uint32_t	cc ;

	while (1)
	{
		// Configure MCO / LSCO
		aaPrintf ("Configure MCO(1) or MCO(2): ") ;
		cc = aaGetChar () ;
		aaPutChar ((char) cc) ;
		aaPutChar ('\n') ;

		if (cc == 'q')
		{
			break ;
		}

		if (cc == '1' || cc =='2')
		{
			configureMco (cc) ;
		}
		else
		{
			aaPrintf ("Unknown command\n") ;
		}
	}
	aaPrintf ("MCO test ended\n") ;
}

//--------------------------------------------------------------------------------
